   title "Roger v.1.1f - created by OK1MPE"

   LIST P=12F629, R=DEC, F=INHX8M
   INCLUDE <P12F629.INC>
   __CONFIG _PWRTE_ON & _WDT_OFF & _MCLRE_OFF & _BODEN_OFF & _INTRC_OSC_NOCLKOUT & _CP_OFF &_CPD_OFF
; ---------------------------------------------------------------------
;
; This software is free for private usage. It was created for HAM radio
; community members. Commercial exploatation is allowed only with
; permission of author.
;
; (C) Petr Bittnar, OK1MPE (ok1mpe@qsl.net), CZ, www.qsl.net/ok1mpe
;                                                www.bittnar.info
;
; ---------------------------------------------------------------------
;
; History:
; 06.02.14 * v.1.1f  - source code was rewritten for 'F' MCU version
;                      !! yes, more then 15 years from init idea :)
; 25.03.01 * v.1.1   - add on/off function	
; 19.03.99 * v.1.0   - first public version
;
; ---------------------------------------------------------------------
;
;**********************************************************************
;*  RC int., Tcpu=1us, 12F629, MPASMWIN 5.20                   6.2.14 *
;**********************************************************************
;
;  Vdd  -| U |- Vss
;  ptti -|   |- D0
;  ptto -|   |- D1
;  D2   -|   |- mod

d0         equ 0          ;def. GPIO
d1         equ 1
d2         equ 3
mod        equ 2
ptto       equ 4
ptti       equ 5

;definition RAM (registers):
freq       equ 20h
delay      equ 21h
temp       equ 22h

;**********************************************************************
;*                        M A C R O                                   *
;**********************************************************************
ton macro freq_,delay_
   movlw   freq_          ;settings parameters (freq and delay)
   movwf   freq           ;
   movlw   delay_         ;
   movwf   delay          ;
   call    gen_ton        ;call subrutine for generation wave
   endm

   org     0              ;reset vector
;**********************************************************************
;* settings ports, prescaler ...                                      *
;**********************************************************************
;ports: 0-output, 1-input
start
   bcf     STATUS,RP0     ;bank 0
   clrf    GPIO           ;init GPIO
   movlw   b'00000111'    ;comparator off, digital pins
   movwf   CMCON          ;
   bsf     STATUS,RP0     ;bank 1
;   clrf    ANSEL          ;digital pins (UNCOMMENT FOR PIC12F675 !!)
   movlw   b'00101011'    ;settings GPIO (input/output)
   movwf   TRISIO
   clrf    INTCON         ;interupts disable
   movlw   b'00000111'    ;ps 1:256, pull-up enable
   movwf   OPTION_REG     ;
   movlw   b'00111011'    ;pull-up
   movwf   WPU            ;ok
   call    3ffh           ;call calibration value
   movwf   OSCCAL         ;calibrate RC osc.
   bcf     STATUS,RP0     ;bank 0

; ------------------------ on/off function ----------------------------
   movlw   .80            ;timer 2s
   movwf   temp           ;(80 x 25)
onoff
   btfsc   GPIO,ptti      ;release PTT key?
   goto    main           ;yes goto main
   call    wait           ;
   decfsz  temp           ;timer overflow?
   goto    onoff          ;no repeat

roger_off
   goto    roger_off      ;yes turn permanently off

; ------------------------ main rutine -------------------------------
main
   btfsc   GPIO,ptti      ;wait for PTT key (press)
   goto    $-1            ;

   call    wait           ;wait a moment

   btfss   GPIO,ptti      ;wait for PTT key (release)
   goto    $-1            ;

   bsf     GPIO,ptto      ;PTT output = H (TX on)
   call    wait           ;
   call    wait           ;wait 100ms
   call    wait           ;
   call    wait           ;

   clrf    temp
   btfss   GPIO,d0        ;read input pins D0-D2
   bsf     temp,0         ;
   btfss   GPIO,d1        ;
   bsf     temp,1         ;
   btfss   GPIO,d2        ;
   bsf     temp,2         ;

   movfw   temp
   addwf   PCL
   goto    typ1           ;typ 1
   goto    typ2           ;typ 2
   goto    typ3           ;typ 3
   goto    typ4           ;typ 4
   goto    typ5           ;typ 5
   goto    typ6           ;typ 6
   goto    typ7           ;typ 7
   goto    typ8           ;typ 8

typ1
   ton     .208,.56       ;800Hz, 70ms
   goto    quit

typ2
   ton     .119,.98       ;1400Hz, 70ms
   goto    quit

typ3
   ton     .83,.141       ;2000Hz, 70ms
   goto    quit

typ4
   ton     .167,.70       ;1000Hz, 70ms
   call    wait           ;
   call    wait           ;wait 75ms
   call    wait           ;
   ton     .167,.70       ;1000Hz, 70ms
   goto    quit

typ5
   ton     .167,.70       ;1000Hz, 70ms
   ton     .83,.141       ;2000Hz, 70ms
   goto    quit

typ6
   ton     .83,.141       ;2000Hz, 70ms
   ton     .167,.70       ;1000Hz, 70ms
   goto    quit

typ7
   ton     .208,.40       ;800Hz, 50ms
   ton     .167,.50       ;1000Hz, 50ms
   ton     .139,.60       ;1200Hz, 50ms
   goto    quit

typ8
   ton     .139,.60       ;1200Hz, 50ms
   ton     .167,.50       ;1000Hz, 50ms
   ton     .208,.40       ;800Hz, 50ms
   goto    quit

quit
   call    wait           ;wait a moment
   bcf     GPIO,ptto      ;PTT output L (TX off)
   call    wait           ;
   goto    main

;**********************************************************************
;*                        S U B R U T I N E S                         *
;**********************************************************************
;subrutine: generation wave
gen_ton
   movfw   freq           ;set temp from freq
   movwf   temp           ;
   bsf     GPIO,mod       ;modulation = H
loop_hi
   decfsz  temp           ;repeat while temp>0
   goto    loop_hi        ;

   movfw   freq           ;set temp from freq
   movwf   temp           ;
   bcf     GPIO,mod       ;modulation = L
loop_lo
   decfsz  temp           ;repeat while temp>0
   goto    loop_lo        ;

   decfsz  delay          ;repeat while delay>0
   goto    gen_ton        ;
   retlw   0

;----------------------------------------------------------------------
;subrutine: wait 25ms
wait
   movlw   .158           ;(256-158)x256us
   movwf   TMR0           ;timer~25ms

   movfw   TMR0           ;read timer
   btfss   STATUS,Z       ;overflow?
   goto    $-2            ;no repeat
   retlw   0              ;

   end
